package com.xinwangdao.CompanyStatistical.annotation;

import com.xinwangdao.CompanyStatistical.entity.User;
import com.xinwangdao.CompanyStatistical.component.GlobalException;
import com.xinwangdao.CompanyStatistical.service.ILoginService;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.UnknownAccountException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

import java.util.Optional;

/**
 * @Author: zthu
 * @Date: 2019/1/9 17:10
 */
public class CurrentUserMethodArgumentResolver implements HandlerMethodArgumentResolver {
    @Autowired
    ILoginService loginService;

    @Override
    public boolean supportsParameter(MethodParameter parameter) {
        return parameter.getParameterType().isAssignableFrom(User.class)
                && parameter.hasParameterAnnotation(CurrentUser.class);
    }

    @Override
    public Object resolveArgument(MethodParameter parameter, ModelAndViewContainer mavContainer, NativeWebRequest webRequest, WebDataBinderFactory binderFactory) throws Exception {
        String userName = (String)SecurityUtils.getSubject().getPrincipal();
        Optional<User> userOptional = loginService.findByName(userName);
        User user = userOptional.orElseThrow(() -> GlobalException.USER_NOT_EXIST.ex());
        return user;
    }
}
