package com.xinwangdao.CompanyStatistical.component;

import com.xinwangdao.CompanyStatistical.component.GlobalException;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.ToString;

/**
 * 统一返回对象
 *
 * @Author: zthu
 * @Date: 2021-06-15
 */
@Data
@ToString
@AllArgsConstructor
@ApiModel("统一返回对象")
public class GlobalResponse<T> {
    @ApiModelProperty("是否成功")
    private boolean success;

    @ApiModelProperty("返回数据")
    private T data;

    @ApiModelProperty("错误码")
    private Long errorCode;

    @ApiModelProperty("错误信息")
    private String errorMsg;

    public static <T> GlobalResponse<T> success() {
        return new GlobalResponse<>(true, null, null, null);
    }

    public static <T> GlobalResponse<T> success(T data) {
        return new GlobalResponse<>(true, data, null, null);
    }

    public static <T> GlobalResponse<T> fail() {
        return new GlobalResponse<>(false, null, null, null);
    }

    public static <T> GlobalResponse<T> fail(String Msg) {
        return new GlobalResponse<T>(false, null, GlobalException.UNKNOWN_ERROR.getCode(), Msg);
    }

    public static <T> GlobalResponse<T> fail(GlobalException exception) {
        return new GlobalResponse<T>(false, null, exception.getCode(), exception.getDesc());
    }

    public static <T> GlobalResponse<T> fail(GlobalException exception, String Msg) {
        return new GlobalResponse<T>(false, null, exception.getCode(), Msg);
    }
}
