package com.xinwangdao.CompanyStatistical.controller;

import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.base.Strings;
import com.xinwangdao.CompanyStatistical.component.GlobalResponse;
import com.xinwangdao.CompanyStatistical.component.QueryPageBean;
import com.xinwangdao.CompanyStatistical.component.ResultPageBean;
import com.xinwangdao.CompanyStatistical.entity.Company;
import com.xinwangdao.CompanyStatistical.entity.CompanyType;
import com.xinwangdao.CompanyStatistical.entity.ProjectCompanyRelation;
import com.xinwangdao.CompanyStatistical.entity.User;
import com.xinwangdao.CompanyStatistical.entity.dto.CompanyDTO;
import com.xinwangdao.CompanyStatistical.mapper.CompanyMapper;
import com.xinwangdao.CompanyStatistical.service.ICompanyService;
import com.xinwangdao.CompanyStatistical.service.ICompanyTypeService;
import com.xinwangdao.CompanyStatistical.service.IProjectCompanyRelationService;
import com.xinwangdao.CompanyStatistical.service.IUserService;
import com.xinwangdao.CompanyStatistical.util.ShiroUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;

import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.fail;
import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.success;

/**
 * 前端控制器
 *
 * @Author: zthu
 * @Date: 2021-06-15
 */
@Api(description = "")
@Slf4j
@RestController
@RequestMapping("/api/admin/company")
public class CompanyController {
    @Autowired
    ICompanyService companyService;

    @Autowired
    ICompanyTypeService companyTypeService;

    @Autowired
    CompanyMapper companyMapper;

    @Autowired
    IUserService userService;

    @Autowired
    IProjectCompanyRelationService projectCompanyRelationService;

    @ApiOperation("根据id获取")
    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public GlobalResponse<Company> getOne(@PathVariable Integer id) {
        return success(companyService.getById(id));
    }

    @ApiOperation("新增")
    @RequestMapping(value = "", method = RequestMethod.POST)
    public GlobalResponse addOne(@RequestBody Company entity) {
        entity.setCompanyTypes("," + entity.getCompanyTypes() + ",");
        companyService.save(entity);
        return success(entity);
    }

    @ApiOperation("修改")
    @RequestMapping(value = "/{id}", method = RequestMethod.PUT)
    public GlobalResponse updateOne(@PathVariable Integer id,
                                    @RequestBody Company entity) {
        companyService.updateById(entity.setId(id));
        return success(entity);
    }

    @ApiOperation("删除")
    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public GlobalResponse deleteOne(@PathVariable Integer id) {
        Company company = companyService.getById(id);
        userService.removeById(company.getId());
        companyService.removeById(id);
        return success();
    }

    @ApiOperation("列表查询")
    @RequestMapping(value = "", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<Company>> list(QueryPageBean queryBean) {
        IPage page = companyService.page(queryBean.toPage(), Wrappers.<Company>lambdaQuery()
                .like(!Strings.isNullOrEmpty(queryBean.getKeyword()), Company::getId, queryBean.getKeyword())
        );
        ResultPageBean<Company> returnVal = new ResultPageBean(page);
        return success(returnVal);
    }

    @ApiOperation("查询企业列表")
    @RequestMapping(value = "/queryCompanyList", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<CompanyDTO>> queryCompanyList(QueryPageBean queryBean,
                                                                       @RequestParam(value = "companyName", required = false) String companyName,
                                                                       @RequestParam(value = "companyType", required = false) String companyType) {
        IPage page = companyService.page(queryBean.toPage(), Wrappers.<Company>lambdaQuery()
                .like(!Strings.isNullOrEmpty(queryBean.getKeyword()), Company::getId, queryBean.getKeyword())
                .like(!Strings.isNullOrEmpty(companyName), Company::getCompanyName, companyName)
                .like(!Strings.isNullOrEmpty(companyType), Company::getCompanyTypes, "," + companyType + ",")
        );

        List<Company> records = page.getRecords();
        ResultPageBean<CompanyDTO> returnVal = new ResultPageBean(page);
        List<CompanyDTO> companyDTOList = new ArrayList<>();
        if (null != records && records.size() > 0) {
            for (int i = 0; i < records.size(); i++) {
                Integer totalNum = companyMapper.selectTaskNum(records.get(i).getId(), null);
                Integer completeNum = companyMapper.selectTaskNum(records.get(i).getId(), 3);
                QueryWrapper<User> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("id", records.get(i).getUserId());
                User user = userService.getOne(queryWrapper);
                CompanyDTO companyDTO = new CompanyDTO();
                BeanUtils.copyProperties(records.get(i), companyDTO);
                companyDTO.setCompletedTaskNum(completeNum);
                companyDTO.setTotalTaskNum(totalNum);
                companyDTO.setUsername(user.getUsername());
                companyDTOList.add(companyDTO);
            }
        }
        returnVal.setRecords(companyDTOList);
        return success(returnVal);
    }

    @ApiOperation("创建企业信息")
    @RequestMapping(value = "/createCompanyInfo", method = RequestMethod.POST)
    public GlobalResponse createCompanyInfo(@RequestBody Company company) {
        company.setCompanyTypes("," + company.getCompanyTypes() + ",");
        companyService.save(company);
        return success();
    }

    @ApiOperation("导入企业信息")
    @RequestMapping(value = "/genCompany", method = RequestMethod.POST)
    public GlobalResponse genCompany(@RequestParam MultipartFile file,@RequestParam(value = "projectId",required = false)String projectId){
        try {
            ExcelReader reader = ExcelUtil.getReader(file.getInputStream());
//        ExcelReader reader = ExcelUtil.getReader("G:\\1.xlsx");
            List<List<Object>> readAll = reader.read();
            ArrayList<Company> companyList = new ArrayList<>();
            HashSet<String> companySet = new HashSet<>();
            HashSet<String> nameSet = new HashSet<>();
            for (int i = 1; i < readAll.size(); i++) {
                List<Object> rowList = readAll.get(i);
                Company company = new Company();
                company.setCreateTime(new Date());
                String username = "";
                for (int j = 0; j < rowList.size(); j++) {
                    switch (j) {
                        case 0:
                            company.setCompanyName(rowList.get(j).toString());
                            break;
                        case 1:
                            company.setBusinessLicenseNo(rowList.get(j).toString());
                            break;
                        case 2:
                            company.setLinkman(rowList.get(j).toString());
                            break;
                        case 3:
                            username = rowList.get(j).toString();
                            company.setLinkPhone(rowList.get(j).toString());
                            break;
                        case 4:
                            company.setCompanyUnit(rowList.get(j).toString());
                            break;
                        case 5:
                            String typeNames = rowList.get(j).toString();
                            String[] types = typeNames.split(",");
                            String type = "";
                            for (int k = 0; k < types.length; k++) {
                                QueryWrapper<CompanyType> queryWrapper = new QueryWrapper<>();
                                queryWrapper.eq("type_name",types[k]);
                                CompanyType companyType = companyTypeService.getOne(queryWrapper);
                                if(null != companyType){
                                    type = type+","+companyType.getId();
                                }
                            }
                            company.setCompanyTypes(type+",");
                            company.setCompanyTypesName(rowList.get(j).toString());
                            break;
                        default:
                            break;

                    }
                }
                //根据营业执照号去重
                if(companySet.contains(company.getBusinessLicenseNo())){
                    continue;
                }

                QueryWrapper<User> userQueryWrapper = new QueryWrapper<>();
                userQueryWrapper.eq("username",username);
                User userOne = userService.getOne(userQueryWrapper);
                if(null != userOne ){
                    return fail("存在重复用户名");
                }

                companySet.add(company.getBusinessLicenseNo());
                QueryWrapper<Company> qw = new QueryWrapper<>();
                qw.eq("business_license_no",company.getBusinessLicenseNo());
                Company one = companyService.getOne(qw);
                if(null != one){
                    if(!Strings.isNullOrEmpty(projectId)){
                        //插入项目数据
                        ProjectCompanyRelation relation = new ProjectCompanyRelation();
                        relation.setCreateTime(new Date());
                        relation.setCompanyId(one.getId());
                        relation.setProjectId(Integer.valueOf(projectId));
                        QueryWrapper<ProjectCompanyRelation> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("company_id",one.getId()).eq("project_id",Integer.valueOf(projectId));
                        ProjectCompanyRelation pcr = projectCompanyRelationService.getOne(queryWrapper);
                        if(null == pcr){
                            projectCompanyRelationService.save(relation);
                        }

                    }
                    continue;
                }

                String salt = ShiroUtils.generateSalt();
                String password = ShiroUtils.md5Password("123456", salt);
                User user = new User().setName(username).setUsername(username)
                        .setSalt(salt)
                        .setPassword(password).setCreateTime(new Date());
                userService.save(user);
                company.setUserId(user.getId());
                companyList.add(company);
                if(!Strings.isNullOrEmpty(projectId)){
                    boolean saveRes = companyService.save(company);
                    if(saveRes){
                        //插入项目数据
                        ProjectCompanyRelation relation = new ProjectCompanyRelation();
                        relation.setCreateTime(new Date());
                        relation.setCompanyId(company.getId());
                        relation.setProjectId(Integer.valueOf(projectId));
                        QueryWrapper<ProjectCompanyRelation> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("company_id",company.getId()).eq("project_id",Integer.valueOf(projectId));
                        ProjectCompanyRelation pcr = projectCompanyRelationService.getOne(queryWrapper);
                        if(null == pcr){
                            projectCompanyRelationService.save(relation);
                        }
                    }
                }
            }
            if(Strings.isNullOrEmpty(projectId)){
                companyService.saveBatch(companyList);
            }

            return success();
        } catch (IOException e) {
            e.printStackTrace();
            return fail();
        }
    }
}
