package com.xinwangdao.CompanyStatistical.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.base.Strings;
import com.xinwangdao.CompanyStatistical.component.GlobalResponse;
import com.xinwangdao.CompanyStatistical.component.QueryPageBean;
import com.xinwangdao.CompanyStatistical.component.ResultPageBean;
import com.xinwangdao.CompanyStatistical.entity.Company;
import com.xinwangdao.CompanyStatistical.entity.CompanyType;
import com.xinwangdao.CompanyStatistical.service.ICompanyService;
import com.xinwangdao.CompanyStatistical.service.ICompanyTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.success;

/**
 *  前端控制器
 *
 * @Author: zthu
 * @Date: 2021-06-15
 */
@Api(description = "")
@Slf4j
@RestController
@RequestMapping("/api/admin/company-type")
public class CompanyTypeController {
    @Autowired
    ICompanyTypeService companyTypeService;

    @Autowired
    ICompanyService companyService;

    @ApiOperation("根据id获取")
    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public GlobalResponse<CompanyType> getOne(@PathVariable Integer id) {
        return success(companyTypeService.getById(id));
    }

    @ApiOperation("新增")
    @RequestMapping(value = "", method = RequestMethod.POST)
    public GlobalResponse addOne(@RequestBody CompanyType entity) {
        companyTypeService.save(entity);
        return success(entity);
    }

    @ApiOperation("修改")
    @RequestMapping(value = "/{id}", method = RequestMethod.PUT)
    public GlobalResponse updateOne(@PathVariable Integer id,
                                                @RequestBody CompanyType entity) {
        companyTypeService.updateById(entity.setId(id));
        return success(entity);
    }

    @ApiOperation("删除")
    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public GlobalResponse deleteOne(@PathVariable Integer id) {
        CompanyType companyType = companyTypeService.getById(id);
        companyTypeService.removeById(id);

        QueryWrapper<Company> queryWrapper = new QueryWrapper<>();
        queryWrapper.like( "company_types", "," + id + ",");
        List<Company> list = companyService.list(queryWrapper);

        for (Company company:list){
            String companyTypes = company.getCompanyTypes();
            String companyTypesName = company.getCompanyTypesName();
            String[] typeSplit = companyTypes.split(",");
            String type = "";
            for (int i = 0; i < typeSplit.length; i++) {
                if(!id.toString().equals(typeSplit[i])){
                    type = type+","+typeSplit[i];
                }
            }

            String[] typeNameSplit = companyTypesName.split(",");
            String typeName = "";
            for (int j = 0; j < typeNameSplit.length; j++) {
                if(!companyType.getTypeName().equals(typeNameSplit[j])){
                    typeName = typeName+","+typeNameSplit[j];
                }
            }

            company.setCompanyTypes(type);
            company.setCompanyTypesName(typeName);
        }
        companyService.saveBatch(list);
        return success();
    }

    @ApiOperation("列表查询")
    @RequestMapping(value = "", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<CompanyType>> list(QueryPageBean queryBean) {
        IPage page = companyTypeService.page(queryBean.toPage(), Wrappers.<CompanyType>lambdaQuery()
                .like(!Strings.isNullOrEmpty(queryBean.getKeyword()), CompanyType::getTypeName, queryBean.getKeyword())
        );
        ResultPageBean<CompanyType> returnVal = new ResultPageBean(page);
        return success(returnVal);
    }
}
