package com.xinwangdao.CompanyStatistical.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xinwangdao.CompanyStatistical.annotation.CurrentUser;
import com.xinwangdao.CompanyStatistical.component.GlobalResponse;
import com.xinwangdao.CompanyStatistical.entity.Company;
import com.xinwangdao.CompanyStatistical.entity.User;
import com.xinwangdao.CompanyStatistical.entity.dto.LoginDTO;
import com.xinwangdao.CompanyStatistical.entity.dto.PasswordDTO;
import com.xinwangdao.CompanyStatistical.service.ICompanyService;
import com.xinwangdao.CompanyStatistical.service.IUserService;
import com.xinwangdao.CompanyStatistical.util.ShiroUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.authz.annotation.RequiresGuest;
import org.apache.shiro.authz.annotation.RequiresUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.fail;
import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.success;

/**
 * @Author: zthu
 * @Date: 2021-06-15
 */
@Api(description = "登录")
@Slf4j
@RestController
@RequestMapping("/api/admin/user")
public class LoginController {
    @Autowired
    IUserService userService;

    @Autowired
    ICompanyService companyService;

    @ApiOperation("注册")
    @RequiresGuest
    @RequestMapping(value = "/register", method = RequestMethod.POST)
    public GlobalResponse register(@Valid @RequestBody LoginDTO loginDTO) {
        QueryWrapper<User> userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.eq("username",loginDTO.getUsername());
        User user1 = userService.getOne(userQueryWrapper);
        if(null != user1){
            return fail("此用户名已被占用");
        }

        String salt = ShiroUtils.generateSalt();
        String password = ShiroUtils.md5Password(loginDTO.getPassword(), salt);
        User user = new User().setName(loginDTO.getUsername()).setUsername(loginDTO.getUsername())
                .setSalt(salt)
                .setPassword(password);
        userService.save(user);
        return success(user);
    }

    @ApiOperation("修改密码")
    @RequiresGuest
    @RequestMapping(value = "/updatePassword", method = RequestMethod.POST)
    public GlobalResponse updatePassword(@Valid @RequestBody PasswordDTO passwordDTO) {
        QueryWrapper<User> userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.eq("username",passwordDTO.getUsername());
        User user1 = userService.getOne(userQueryWrapper);
        if(null == user1){
            return fail("用户名或密码不正确");
        }

        String password = ShiroUtils.md5Password(passwordDTO.getOldPassword(), user1.getSalt());
        if(!password.equals(user1.getPassword())){
            return fail("用户名或密码不正确");
        }
        String newPassword = ShiroUtils.md5Password(passwordDTO.getNewPassword(), user1.getSalt());
        user1.setPassword(newPassword);
        userService.updateById(user1);
        return success();
    }

    @ApiOperation("重置密码")
    @RequestMapping(value = "/resetPassword", method = RequestMethod.POST)
    public GlobalResponse resetPassword(@RequestBody PasswordDTO passwordDTO) {
        QueryWrapper<User> userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.eq("username",passwordDTO.getUsername());
        User user1 = userService.getOne(userQueryWrapper);
        if(null == user1){
            return fail("用户名不存在");
        }

        String newPassword = ShiroUtils.md5Password("123456", user1.getSalt());
        user1.setPassword(newPassword);
        userService.updateById(user1);
        return success();
    }

    @ApiOperation("登录")
    @RequestMapping(value = "/login", method = RequestMethod.POST)
    public GlobalResponse login(@Valid @RequestBody LoginDTO loginDTO, @RequestParam(required = false, defaultValue = "false") Boolean rememberMe) {
        UsernamePasswordToken token = new UsernamePasswordToken(loginDTO.getUsername(), loginDTO.getPassword());
        token.setRememberMe(rememberMe);
        SecurityUtils.getSubject().login(token);
        QueryWrapper<User> userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.eq("username",loginDTO.getUsername());
        User user = userService.getOne(userQueryWrapper);
        if(null == user || 0 == user.getStatus()){
            SecurityUtils.getSubject().logout();
            return fail("该用户已被禁用");
        }

        QueryWrapper<Company> companyQueryWrapper = new QueryWrapper<>();
        companyQueryWrapper.eq("user_id",user.getId());
        Company company = companyService.getOne(companyQueryWrapper);

        if(null != company){
            company.setLastLoginTime(new Date());
            companyService.updateById(company);
            if(0 == company.getStatus()){
                SecurityUtils.getSubject().logout();
                return fail("该企业已被禁用");
            }
        }

        Map map = new HashMap<String,Object>();
        map.put("user",user);
        map.put("cookie", SecurityUtils.getSubject().getSession().getId());
        map.put("time",System.currentTimeMillis()+(long) (1000*60 * 60 * 24 * 7));
        map.put("companyId",null == company?"":company.getId());
        return success(map);

    }

    @ApiOperation("登出")
    @RequiresUser
    @RequestMapping(value = "/logout", method = RequestMethod.POST)
    public GlobalResponse logout() {
        SecurityUtils.getSubject().logout();
        return success();
    }

    @ApiOperation("当前用户信息")
    @RequiresUser
    @RequestMapping(value = "/info", method = RequestMethod.GET)
    public GlobalResponse info(@ApiIgnore @CurrentUser User user) {

        return success(new HashMap<String, Object>() {
            {
                put("name", "Admin");
                put("avatar", "https://wpimg.wallstcn.com/f778738c-e4f8-4870-b634-56703b4acafe.gif");
                put("introduction", "I am a super administrator");
                put("roles", new String[]{"admin"});
            }
        });
    }
}
