package com.xinwangdao.CompanyStatistical.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.Strings;
import com.xinwangdao.CompanyStatistical.component.GlobalResponse;
import com.xinwangdao.CompanyStatistical.component.QueryPageBean;
import com.xinwangdao.CompanyStatistical.component.ResultPageBean;
import com.xinwangdao.CompanyStatistical.entity.*;
import com.xinwangdao.CompanyStatistical.entity.dto.ProjectByCompanyIdDTO;
import com.xinwangdao.CompanyStatistical.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.fail;
import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.success;

/**
 *  前端控制器
 *
 * @Author: zthu
 * @Date: 2021-06-15
 */
@Api(description = "")
@Slf4j
@RestController
@RequestMapping("/api/admin/project-company-relation")
public class ProjectCompanyRelationController {
    @Autowired
    IProjectCompanyRelationService projectCompanyRelationService;

    @Autowired
    ICompanyService companyService;

    @Autowired
    IProjectDetailService projectDetailService;

    @Autowired
    IDetailLogService detailLogService;

    @Autowired
    ISatisfactionLogService satisfactionLogService;

    @Autowired
    IProjectService projectService;

    @ApiOperation("根据id获取")
    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public GlobalResponse<ProjectCompanyRelation> getOne(@PathVariable Integer id) {
        return success(projectCompanyRelationService.getById(id));
    }

    @ApiOperation("新增")
    @RequestMapping(value = "", method = RequestMethod.POST)
    public GlobalResponse addOne(@RequestBody ProjectCompanyRelation entity) {
        QueryWrapper<ProjectCompanyRelation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("company_id",entity.getCompanyId()).eq("project_id",entity.getProjectId());
        ProjectCompanyRelation one = projectCompanyRelationService.getOne(queryWrapper);
        if(null != one){
            return fail("此企业已添加，请勿重复添加");
        }

        projectCompanyRelationService.save(entity);
        return success(entity);
    }

    @ApiOperation("修改")
    @RequestMapping(value = "/{id}", method = RequestMethod.PUT)
    public GlobalResponse updateOne(@PathVariable Integer id,
                                                @RequestBody ProjectCompanyRelation entity) {
        ProjectCompanyRelation projectCompanyRelation = projectCompanyRelationService.getById(id);
        Project project = projectService.getById(projectCompanyRelation.getProjectId());
        if(1 != project.getStatus()){
            return fail("项目已停用或已截至");
        }
        if(2==entity.getStatus()){
            ProjectCompanyRelation pcr = projectCompanyRelationService.getById(id);
            QueryWrapper<ProjectDetail> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("project_id",pcr.getProjectId()).eq("status",1);
            List<ProjectDetail> projectDetailList = projectDetailService.list(queryWrapper);
            for (ProjectDetail projectDetail:projectDetailList){
                if(1 == projectDetail.getType()){
                    QueryWrapper<DetailLog> qw = new QueryWrapper<>();
                    qw.eq("detail_id",projectDetail.getId()).eq("company_id",pcr.getCompanyId()).eq("log_type",1);
                    DetailLog detailLog = detailLogService.getOne(qw);
                    if(null == detailLog || null == detailLog.getStatus() || 0==detailLog.getStatus()){
                        return fail("请先完成全部表单");
                    }
                }else if(2 == projectDetail.getType()){
                    QueryWrapper<DetailLog> qw = new QueryWrapper<>();
                    qw.eq("detail_id",projectDetail.getId()).eq("company_id",pcr.getCompanyId()).eq("log_type",2);
                    DetailLog detailLog = detailLogService.getOne(qw);
                    if(null == detailLog || null == detailLog.getStatus() || 0==detailLog.getStatus()){
                        return fail("请先完成全部表单");
                    }
                }else if(3 == projectDetail.getType()){
                    QueryWrapper<SatisfactionLog> qw = new QueryWrapper<>();
                    qw.eq("satisfaction_id",projectDetail.getSatisfactionId()).eq("company_id",pcr.getCompanyId()).eq("project_id",pcr.getProjectId());
                    SatisfactionLog satisfactionLog = satisfactionLogService.getOne(qw);
                    if(null == satisfactionLog || null == satisfactionLog.getStatus() || 0==satisfactionLog.getStatus()){
                        return fail("请先完成全部表单");
                    }
                }

            }
        }
        projectCompanyRelationService.updateById(entity.setId(id));
        return success(entity);
    }

    @ApiOperation("删除项目中的企业")
    @RequestMapping(value = "/removeCompanyInProject", method = RequestMethod.DELETE)
    public GlobalResponse removeCompanyInProject(@RequestParam("projectId")Integer projectId,@RequestParam("companyId")Integer companyId) {
        QueryWrapper<ProjectCompanyRelation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("company_id",companyId).eq("project_id",projectId);
        ProjectCompanyRelation one = projectCompanyRelationService.getOne(queryWrapper);
        if(null != one){
            projectCompanyRelationService.remove(queryWrapper);
        }
        return success();
    }

    @ApiOperation("删除")
    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public GlobalResponse deleteOne(@PathVariable Integer id) {
        projectCompanyRelationService.removeById(id);
        return success();
    }

    @ApiOperation("列表查询")
    @RequestMapping(value = "", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<ProjectCompanyRelation>> list(QueryPageBean queryBean) {
        IPage page = projectCompanyRelationService.page(queryBean.toPage(), Wrappers.<ProjectCompanyRelation>lambdaQuery()
                .like(!Strings.isNullOrEmpty(queryBean.getKeyword()), ProjectCompanyRelation::getId, queryBean.getKeyword())
        );
        ResultPageBean<ProjectCompanyRelation> returnVal = new ResultPageBean(page);
        return success(returnVal);
    }

    @ApiOperation("根据企业id查询项目列表")
    @RequestMapping(value = "/selectProjectListByCompanyId", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<ProjectByCompanyIdDTO>> selectProjectListByCompanyId(QueryPageBean queryBean,@RequestParam(value = "companyId")Integer companyId,
                                                                                              @RequestParam(value = "status",required = false)Integer status,
                                                                                              @RequestParam(value = "applyStatus",required = false)Integer applyStatus,
                                                                                              @RequestParam(value = "projectName",required = false)String projectName) {
        Page<ProjectCompanyRelation> page1 = new Page<>(queryBean.getCurrent(), queryBean.getSize());
        IPage page = projectCompanyRelationService.selectProjectListByCompanyId(page1,companyId,status,applyStatus,Strings.isNullOrEmpty(projectName)?null:projectName);
        ResultPageBean<ProjectByCompanyIdDTO> returnVal = new ResultPageBean(page);
        return success(returnVal);
    }

    @ApiOperation("根据企业id项目id查询关联表")
    @RequestMapping(value = "/selectRelationByCompanyId", method = RequestMethod.GET)
    public GlobalResponse<ProjectCompanyRelation> selectRelationByCompanyId(QueryPageBean queryBean,@RequestParam(value = "companyId")Integer companyId,
                                                                                            @RequestParam(value = "projectId")Integer projectId,
                                                                                              @RequestParam(value = "status",required = false)Integer status) {
        QueryWrapper<ProjectCompanyRelation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("company_id",companyId).eq("project_id",projectId).eq(null != status,"status",status);
        ProjectCompanyRelation one = projectCompanyRelationService.getOne(queryWrapper);
        return success(one);
    }

    @ApiOperation("同一个项目id添加同一分类下所有企业")
    @RequestMapping(value = "/insertRelationList", method = RequestMethod.GET)
    public GlobalResponse<ProjectCompanyRelation> insertRelationList(QueryPageBean queryBean,@RequestParam(value = "companyType")Integer companyType,
                                                                            @RequestParam(value = "projectId")Integer projectId) {
        QueryWrapper<Company> queryWrapper = new QueryWrapper<>();
        queryWrapper.like("company_types",","+companyType+",").eq("status",1);
        List<Company> companyList = companyService.list(queryWrapper);
        QueryWrapper<ProjectCompanyRelation> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("project_id",projectId);
        List<ProjectCompanyRelation> list1 = projectCompanyRelationService.list(queryWrapper1);
        if(null != companyList && companyList.size()>0){
            ArrayList<ProjectCompanyRelation> list = new ArrayList<>();
            for (Company company:companyList){
                boolean flag = false;
                for(ProjectCompanyRelation relation:list1){
                    if(relation.getCompanyId().intValue() == company.getId().intValue()){
                        flag = true;
                        break;
                    }
                }
                if(flag){
                    continue;
                }
                ProjectCompanyRelation projectCompanyRelation = new ProjectCompanyRelation();
                projectCompanyRelation.setCompanyId(company.getId());
                projectCompanyRelation.setProjectId(projectId);
                projectCompanyRelation.setStatus(0);
                projectCompanyRelation.setCreateTime(new Date());
                list.add(projectCompanyRelation);
            }
            projectCompanyRelationService.saveBatch(list);
        }
        return success();
    }
}
