package com.xinwangdao.CompanyStatistical.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.Strings;
import com.xinwangdao.CompanyStatistical.component.GlobalResponse;
import com.xinwangdao.CompanyStatistical.component.QueryPageBean;
import com.xinwangdao.CompanyStatistical.component.ResultPageBean;
import com.xinwangdao.CompanyStatistical.entity.*;
import com.xinwangdao.CompanyStatistical.entity.dto.*;
import com.xinwangdao.CompanyStatistical.mapper.ProjectMapper;
import com.xinwangdao.CompanyStatistical.service.*;
import com.xinwangdao.CompanyStatistical.util.FileUtil;
import com.xinwangdao.CompanyStatistical.util.GenerateExcel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.util.*;

import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.success;

/**
 * 前端控制器
 *
 * @Author: zthu
 * @Date: 2021-06-15
 */
@Api(description = "")
@Slf4j
@RestController
@RequestMapping("/api/admin/project")
public class ProjectController {
    @Autowired
    IProjectService projectService;

    @Autowired
    IProjectDetailService projectDetailService;

    @Autowired
    IProjectCompanyRelationService projectCompanyRelationService;

    @Autowired
    ICompanyService companyService;

    @Autowired
    IDetailLogService detailLogService;

    @Autowired
    ISatisfactionLogService satisfactionLogService;

    @Autowired
    ISatisfactionModelService satisfactionModelService;

    @Autowired
    private ProjectMapper projectMapper;

    @ApiOperation("根据id获取")
    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public GlobalResponse<Project> getOne(@PathVariable Integer id) {
        return success(projectService.getById(id));
    }

    @ApiOperation("新增")
    @RequestMapping(value = "", method = RequestMethod.POST)
    public GlobalResponse addOne(@RequestBody Project entity) {
        entity.setCreateTime(new Date());
        projectService.save(entity);
        return success(entity);
    }

    @ApiOperation("修改")
    @RequestMapping(value = "/{id}", method = RequestMethod.PUT)
    public GlobalResponse updateOne(@PathVariable Integer id,
                                    @RequestBody Project entity) {
        projectService.updateById(entity.setId(id));
        return success(entity);
    }

    @ApiOperation("删除")
    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public GlobalResponse deleteOne(@PathVariable Integer id) {
        projectService.removeById(id);
        return success();
    }

    @ApiOperation("列表查询")
    @RequestMapping(value = "", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<Project>> list(QueryPageBean queryBean) {
        IPage page = projectService.page(queryBean.toPage(), Wrappers.<Project>lambdaQuery()
                .like(!Strings.isNullOrEmpty(queryBean.getKeyword()), Project::getId, queryBean.getKeyword())
        );
        ResultPageBean<Project> returnVal = new ResultPageBean(page);
        return success(returnVal);
    }

    @ApiOperation("查询项目列表")
    @RequestMapping(value = "/queryProjectList", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<ProjectDTO>> queryProjectList(QueryPageBean queryBean,
                                                                       @RequestParam(value = "projectName", required = false) String projectName,
                                                                       @RequestParam(value = "projectStatus", required = false) Integer projectStatus,
                                                                       @RequestParam(value = "fillStatus", required = false) Integer fillStatus,
                                                                       @RequestParam(value = "year", required = false) String year) {
        IPage page = projectService.page(queryBean.toPage(), Wrappers.<Project>lambdaQuery()
                .like(!Strings.isNullOrEmpty(queryBean.getKeyword()), Project::getId, queryBean.getKeyword())
                .like(!Strings.isNullOrEmpty(projectName), Project::getProjectName, projectName)
                .eq(null != projectStatus, Project::getStatus, projectStatus)
                .between(!Strings.isNullOrEmpty(year), Project::getCreateTime, year + "-01-01 00:00:00", year + "-12-31 23:59:59")
        );

        List<Project> records = page.getRecords();
        ResultPageBean<ProjectDTO> returnVal = new ResultPageBean(page);
        List<ProjectDTO> projectDTOList = new ArrayList<>();
        if (null != records && records.size() > 0) {
            for (int i = 0; i < records.size(); i++) {
                Integer totalNum = projectMapper.selectTaskNum(records.get(i).getId(), null);
                Integer completeNum = projectMapper.selectTaskNum(records.get(i).getId(), 3);
                Integer unCheckNum = projectMapper.selectTaskNum(records.get(i).getId(), 2);
                ProjectDTO projectDTO = new ProjectDTO();
                BeanUtils.copyProperties(records.get(i), projectDTO);
                projectDTO.setCompleteNum(completeNum);
                projectDTO.setTotalNum(totalNum);
                projectDTO.setUnCheckNum(unCheckNum);
                if (null != fillStatus) {
                    if (1 == fillStatus && totalNum.intValue() == completeNum.intValue()) {
                        //已完成
                        projectDTOList.add(projectDTO);
                    } else if (-1 == fillStatus && totalNum > completeNum) {
                        //未完成
                        projectDTOList.add(projectDTO);
                    }
                } else {
                    projectDTOList.add(projectDTO);
                }

            }
        }
        returnVal.setRecords(projectDTOList);
        return success(returnVal);
    }

    @ApiOperation("根据项目id查询企业列表")
    @RequestMapping(value = "/queryCompanyListByProjectId", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<CompanyProjectDTO>> queryCompanyListByProjectId(QueryPageBean queryBean,
                                                                                         @RequestParam(value = "projectId") Integer projectId,
                                                                                         @RequestParam(value = "companyName", required = false) String companyName,
                                                                                         @RequestParam(value = "companyType", required = false) String companyType, @RequestParam(value = "status", required = false) Integer status) {
        Page<ProjectCompanyRelation> applyLogPage = new Page<>(queryBean.getCurrent(), queryBean.getSize());
        IPage page = projectCompanyRelationService.selectCompanyListByProjectId(applyLogPage, projectId,
                null == companyName || "".equals(companyName) ? null : companyName,
                null == companyType || "".equals(companyType) ? null : companyType,
                null == status ? null : status);


        ResultPageBean<CompanyProjectDTO> returnVal = new ResultPageBean(page);
        return success(returnVal);
    }

    @ApiOperation("根据项目id查询单个项目详情")
    @RequestMapping(value = "/queryProjectDetailByProjectId", method = RequestMethod.GET)
    public GlobalResponse<ProjectDTO> queryProjectDetailByProjectId(QueryPageBean queryBean,
                                                                    @RequestParam(value = "projectId") Integer projectId) {
        Project project = projectService.getById(projectId);
        Integer totalNum = projectMapper.selectTaskNum(project.getId(), null);
        Integer completeNum = projectMapper.selectTaskNum(project.getId(), 3);
        ProjectDTO projectDTO = new ProjectDTO();
        BeanUtils.copyProperties(project, projectDTO);
        projectDTO.setCompleteNum(completeNum);
        projectDTO.setTotalNum(totalNum);

        return success(projectDTO);
    }

    @ApiOperation("导出项目")
    @RequestMapping(value = "/genExcel", method = RequestMethod.GET)
    public GlobalResponse genExcel(HttpServletResponse response, @RequestParam(value = "projectId") Integer projectId) {
        QueryWrapper<ProjectDetail> projectDetailQueryWrapper = new QueryWrapper<>();
        projectDetailQueryWrapper.eq("project_id", projectId);
        List<ProjectDetail> projectDetailList = projectDetailService.list(projectDetailQueryWrapper);

        Project project = projectService.getById(projectId);

        QueryWrapper<ProjectCompanyRelation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("project_id", projectId);
        List<ProjectCompanyRelation> companyRelationList = projectCompanyRelationService.list(queryWrapper);
        HSSFWorkbook workbook = new HSSFWorkbook();
        for (ProjectDetail projectDetail : projectDetailList) {

            ArrayList<ExcelDTO> list = new ArrayList<>();
            List<String> titleList = new ArrayList<String>();
            titleList.add("序号");
            titleList.add("企业名称");
            Boolean flag = true;

            if (2 == projectDetail.getType()) {
                continue;
            }

            for (ProjectCompanyRelation relation : companyRelationList) {
                String content = "";
                if (1 == projectDetail.getType()) {
                    //自定义表单
                    QueryWrapper<DetailLog> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.eq("company_id", relation.getCompanyId()).eq("detail_id", projectDetail.getId()).eq("status", 1);
                    DetailLog detailLog = detailLogService.getOne(queryWrapper1);
                    if (null == detailLog) {
                        continue;
                    }
                    content = detailLog.getContent();
                } else if (3 == projectDetail.getType()) {
                    //满意度调查表
                    QueryWrapper<SatisfactionLog> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.eq("company_id", relation.getCompanyId()).eq("project_id", projectId).eq("status", 1).eq("satisfaction_id", projectDetail.getSatisfactionId());
                    SatisfactionLog satisfactionLog = satisfactionLogService.getOne(queryWrapper1);
                    if (null == satisfactionLog) {
                        continue;
                    }
                    content = satisfactionLog.getContent();
                }


                Company company = companyService.getById(relation.getCompanyId());
                ExcelDTO excelDTO = new ExcelDTO();
                excelDTO.setCompanyName(company.getCompanyName());

                System.out.println(content);
                if (Strings.isNullOrEmpty(content)) {
                    continue;
                }
                JSONObject jsonObject = JSONObject.parseObject(content);
                JSONArray formDataList = jsonObject.getJSONArray("formDataList");
                JSONObject model = jsonObject.getJSONObject("model");
                HashMap<String, String> map = new HashMap<>();
                if (null != formDataList && formDataList.size() > 0) {
                    for (int i = 0; i < formDataList.size(); i++) {
                        JSONObject item = formDataList.getJSONObject(i);
                        if ("Alert".equals(item.getString("name"))
                                || "Link".equals(item.getString("name"))
                                || "Divider".equals(item.getString("name"))) {
                            continue;
                        }
                        String key = item.getString("label");
                        if ("栅格布局".equals(key)) {
                            JSONObject setting = item.getJSONObject("setting");
                            JSONArray colList = setting.getJSONArray("colList");
                            for (int j = 0; j < colList.size(); j++) {
                                JSONObject col = colList.getJSONObject(j);
                                if(null == col){
                                    continue;
                                }
                                if ("Alert".equals(col.getString("name"))
                                        || "Link".equals(col.getString("name"))
                                        || "Divider".equals(col.getString("name"))) {
                                    continue;
                                }
                                String colKey = col.getString("label");
                                String fieldId = col.getString("fieldId");
                                boolean multiple = col.getBooleanValue("multiple");
                                String value = "";
                                if (multiple) {
                                    //多选框
                                    JSONArray jsonArray = model.getJSONArray(fieldId);
                                    if (null != jsonArray && jsonArray.size() > 0) {
                                        for (int k = 0; k < jsonArray.size(); k++) {
                                            value = value + jsonArray.getString(k) + ",";
                                        }
                                    }

                                    value = value.substring(0, value.length() - 1);
                                } else {
                                    //单选框
                                    value = model.getString(fieldId);
                                }

                                map.put(colKey, value);
                                if (!titleList.contains(colKey)) {
                                    titleList.add(colKey);
                                }

                            }
//                            flag = false;

                        } else {
                            String fieldId = item.getString("fieldId");
                            boolean multiple = item.getBooleanValue("multiple");
                            String value = "";
                            if (multiple) {
                                //多选框
                                JSONArray jsonArray = model.getJSONArray(fieldId);
                                if (null != jsonArray && jsonArray.size() > 0) {
                                    for (int j = 0; j < jsonArray.size(); j++) {
                                        value = value + jsonArray.getString(j) + ",";
                                    }
                                }

                                value = value.substring(0, value.length() - 1);
                            } else {
                                //单选框
                                value = model.getString(fieldId);
                            }

                            map.put(key, value);
                            if (!titleList.contains(key)) {
                                titleList.add(key);
                            }

                        }
//                        flag = false;
                    }
                }

                excelDTO.setMap(map);
                excelDTO.setDetailName(projectDetail.getDetailName());
                list.add(excelDTO);
            }

            //一个表单生成一个excel的sheet
            GenerateExcel.exportExcelPaper(workbook, titleList, list, projectDetail.getDetailName());

        }
        try {
            OutputStream output;
            output = response.getOutputStream();
            //清空缓存
            response.reset();

            //定义浏览器响应表头，顺带定义下载名，比如students(中文名需要转义)
            response.setHeader("Content-disposition", "attachment;filename=" + new String(project.getProjectName().getBytes(), "iso-8859-1") + ".xls");
            //定义下载的类型，标明是excel文件
            response.setContentType("application/vnd.ms-excel");
            //这时候把创建好的excel写入到输出流
            workbook.write(output);
            //养成好习惯，出门记得随手关门
            output.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return success();
    }

    @ApiOperation("下载文件")
    @RequestMapping(value = "/downloadFiles", method = RequestMethod.GET)
    public GlobalResponse downloadFiles(HttpServletResponse response, @RequestParam(value = "projectId") Integer projectId) throws Exception {
        Project project = projectService.getById(projectId);
        String[] srcDir = {"/home/server/CompanyStatistical/upload-CompanyStatistical/" + projectId};
        FileUtil.downloadZipFiles(response, srcDir, "temp.zip", project.getProjectName());

        return success();
    }

    @ApiOperation("导出满意度表单列表")
    @RequestMapping(value = "/genSatisfaction", method = RequestMethod.GET)
    public GlobalResponse genSatisfaction(HttpServletResponse response, @RequestParam(value = "satisfactionId") Integer satisfactionId) {
        QueryWrapper<ProjectDetail> projectDetailQueryWrapper = new QueryWrapper<>();
        projectDetailQueryWrapper.eq("satisfaction_id", satisfactionId);
        List<ProjectDetail> projectDetailList = projectDetailService.list(projectDetailQueryWrapper);

        SatisfactionModel satisfactionModel = satisfactionModelService.getById(satisfactionId);


        HSSFWorkbook workbook = new HSSFWorkbook();
        ArrayList<ExcelDTO> list = new ArrayList<>();
        List<String> titleList = new ArrayList<String>();
        for (ProjectDetail projectDetail : projectDetailList) {

//            ArrayList<ExcelDTO> list = new ArrayList<>();
//            List<String> titleList = new ArrayList<String>();
            titleList.add("序号");
            titleList.add("企业名称");
            Boolean flag = true;


            QueryWrapper<ProjectCompanyRelation> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("project_id", projectDetail.getProjectId());
            List<ProjectCompanyRelation> companyRelationList = projectCompanyRelationService.list(queryWrapper);

            for (ProjectCompanyRelation relation : companyRelationList) {
                String content = "";

                //满意度调查表
                QueryWrapper<SatisfactionLog> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("company_id", relation.getCompanyId()).eq("project_id", projectDetail.getProjectId()).eq("status", 1).eq("satisfaction_id", satisfactionId);
                SatisfactionLog satisfactionLog = satisfactionLogService.getOne(queryWrapper1);
                if (null == satisfactionLog) {
                    continue;
                }
                content = satisfactionLog.getContent();


                Company company = companyService.getById(relation.getCompanyId());
                ExcelDTO excelDTO = new ExcelDTO();
                excelDTO.setCompanyName(company.getCompanyName());

                System.out.println(content);
                if (Strings.isNullOrEmpty(content)) {
                    continue;
                }
                JSONObject jsonObject = JSONObject.parseObject(content);
                JSONArray formDataList = jsonObject.getJSONArray("formDataList");
                JSONObject model = jsonObject.getJSONObject("model");
                HashMap<String, String> map = new HashMap<>();
                if (null != formDataList && formDataList.size() > 0) {
                    for (int i = 0; i < formDataList.size(); i++) {
                        JSONObject item = formDataList.getJSONObject(i);
                        if ("Alert".equals(item.getString("name"))
                                || "Link".equals(item.getString("name"))
                                || "Divider".equals(item.getString("name"))) {
                            continue;
                        }
                        String key = item.getString("label");
                        String fieldId = item.getString("fieldId");
                        boolean multiple = item.getBooleanValue("multiple");
                        String value = "";
                        if (multiple) {
                            //多选框
                            JSONArray jsonArray = model.getJSONArray(fieldId);
                            if (null != jsonArray && jsonArray.size() > 0) {
                                for (int j = 0; j < jsonArray.size(); j++) {
                                    value = value + jsonArray.getString(j) + ",";
                                }
                            }

                            value = value.substring(0, value.length() - 1);
                        } else {
                            //单选框
                            value = model.getString(fieldId);
                        }

                        map.put(key, value);
                        if (flag) {
                            titleList.add(key);
                        }

                    }
                    flag = false;
                }

                excelDTO.setMap(map);
                excelDTO.setDetailName(projectDetail.getDetailName());
                list.add(excelDTO);
            }

            //一个表单生成一个excel的sheet
//            GenerateExcel.exportExcelPaper(workbook,titleList,list,projectDetail.getDetailName());

        }
        GenerateExcel.exportExcelPaper(workbook, titleList, list, "sheet");
        try {
            OutputStream output;
            output = response.getOutputStream();
            //清空缓存
            response.reset();

            //定义浏览器响应表头，顺带定义下载名，比如students(中文名需要转义)
            response.setHeader("Content-disposition", "attachment;filename=" + new String(satisfactionModel.getSatisfactionName().getBytes(), "iso-8859-1") + ".xls");
            //定义下载的类型，标明是excel文件
            response.setContentType("application/vnd.ms-excel");
            //这时候把创建好的excel写入到输出流
            workbook.write(output);
            //养成好习惯，出门记得随手关门
            output.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return success();
    }


    @ApiOperation("表单统计数据")
    @RequestMapping(value = "/queryStatistical", method = RequestMethod.GET)
    public GlobalResponse queryStatistical(@RequestParam(value = "projectId") Integer projectId) {
        QueryWrapper<ProjectDetail> projectDetailQueryWrapper = new QueryWrapper<>();
        projectDetailQueryWrapper.eq("project_id", projectId);
        List<ProjectDetail> projectDetailList = projectDetailService.list(projectDetailQueryWrapper);

        QueryWrapper<ProjectCompanyRelation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("project_id", projectId);
        List<ProjectCompanyRelation> companyRelationList = projectCompanyRelationService.list(queryWrapper);

        ArrayList<StatisticalDTO> list = new ArrayList<>();
        for (ProjectDetail projectDetail : projectDetailList) {
            StatisticalDTO statisticalDTO = new StatisticalDTO();
            statisticalDTO.setProjectDetailId(projectDetail.getId());
            statisticalDTO.setProjectDetailName(projectDetail.getDetailName());
            if (2 == projectDetail.getType() || 3 == projectDetail.getType()) {
                continue;
            }
            ArrayList<StatisticalVO> statisticalVOList = new ArrayList<>();
            for (ProjectCompanyRelation relation : companyRelationList) {
                String content = "";
                if (1 == projectDetail.getType()) {
                    //自定义表单
                    QueryWrapper<DetailLog> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.eq("company_id", relation.getCompanyId()).eq("detail_id", projectDetail.getId()).eq("status", 1);
                    DetailLog detailLog = detailLogService.getOne(queryWrapper1);
                    if (null == detailLog) {
                        continue;
                    }
                    content = detailLog.getContent();
                } else if (3 == projectDetail.getType()) {
                    //满意度调查表
                    QueryWrapper<SatisfactionLog> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.eq("company_id", relation.getCompanyId()).eq("project_id", projectId).eq("status", 1).eq("satisfaction_id", projectDetail.getSatisfactionId());
                    SatisfactionLog satisfactionLog = satisfactionLogService.getOne(queryWrapper1);
                    if (null == satisfactionLog) {
                        continue;
                    }
                    content = satisfactionLog.getContent();
                }

                System.out.println(content);
                if (Strings.isNullOrEmpty(content)) {
                    continue;
                }
                JSONObject jsonObject = JSONObject.parseObject(content);
                JSONArray formDataList = jsonObject.getJSONArray("formDataList");
                JSONObject model = jsonObject.getJSONObject("model");
                if (null != formDataList && formDataList.size() > 0) {
                    for (int i = 0; i < formDataList.size(); i++) {
                        JSONObject item = formDataList.getJSONObject(i);
                        if ("Alert".equals(item.getString("name"))
                                || "Link".equals(item.getString("name"))
                                || "Divider".equals(item.getString("name"))) {
                            continue;
                        }
                        String key = item.getString("label");
                        if ("栅格布局".equals(key)) {
                            JSONObject setting = item.getJSONObject("setting");
                            JSONArray colList = setting.getJSONArray("colList");
                            for (int j = 0; j < colList.size(); j++) {
                                JSONObject col = colList.getJSONObject(j);
                                if(null == col){
                                    continue;
                                }
                                String colKey = col.getString("label");
                                String fieldId = col.getString("fieldId");
                                String statistics = col.getString("statistics");
                                if (!Strings.isNullOrEmpty(statistics)) {
                                    String value = model.getString(fieldId);
                                    StatisticalVO statisticalVO = new StatisticalVO();
                                    statisticalVO.setKey(colKey);
                                    statisticalVO.setValue(value);
                                    statisticalVO.setStatistics(statistics);
                                    statisticalVOList.add(statisticalVO);
                                }
                            }
                        }else {
                            String fieldId = item.getString("fieldId");
                            String statistics = item.getString("statistics");

                            if (!Strings.isNullOrEmpty(statistics)) {
                                String value = model.getString(fieldId);
                                StatisticalVO statisticalVO = new StatisticalVO();
                                statisticalVO.setKey(key);
                                statisticalVO.setValue(value);
                                statisticalVO.setStatistics(statistics);
                                statisticalVOList.add(statisticalVO);
                            }
                        }

                    }
                }
            }
            String detailContent = projectDetail.getDetailContent();
            JSONObject jsonObject = JSONObject.parseObject(detailContent);
            JSONArray formDataList = jsonObject.getJSONArray("formDataList");


            HashMap<String, List<String>> hashMap = new HashMap<>();
            HashMap<String, String> map = new HashMap<>();
            for (StatisticalVO statisticalVO : statisticalVOList) {
                String key = statisticalVO.getKey();
                if (hashMap.containsKey(key)) {
                    hashMap.get(key).add(statisticalVO.getValue());
                } else {
                    if (null != formDataList && formDataList.size() > 0) {
                        for (int i = 0; i < formDataList.size(); i++) {
                            JSONObject item = formDataList.getJSONObject(i);
                            if ("Alert".equals(item.getString("name"))
                                    || "Link".equals(item.getString("name"))
                                    || "Divider".equals(item.getString("name"))) {
                                continue;
                            }
                            String label = item.getString("label");

                            if ("栅格布局".equals(label)) {
                                JSONObject setting = item.getJSONObject("setting");
                                JSONArray colList = setting.getJSONArray("colList");
                                for (int j = 0; j < colList.size(); j++) {
                                    JSONObject col = colList.getJSONObject(j);
                                    if(null == col){
                                        continue;
                                    }
                                    String colKey = col.getString("label");
                                    String statistics = col.getString("statistics");
                                    if (!Strings.isNullOrEmpty(statistics)) {
                                        if (key.equals(colKey)) {
                                            map.put(colKey, statistics);
                                        }
                                    }
                                }
                            }else {
                                String statistics = item.getString("statistics");

                                if (!Strings.isNullOrEmpty(statistics)) {
                                    if (key.equals(label)) {
                                        map.put(key, statistics);
                                    }
                                }
                            }
                        }
                    }

                    ArrayList<String> valueStrings = new ArrayList<>();
                    valueStrings.add(statisticalVO.getValue());
                    hashMap.put(key, valueStrings);
                    /*map.put(key,statisticalVO.getStatistics());*/
                }
            }
            List<Map<String, String>> result = new ArrayList<>();
            for (String key : map.keySet()) {
                HashMap<String, String> resultMap = new HashMap<>();
                List<String> valueList = hashMap.get(key);
                String statistics = map.get(key);
                resultMap.put("label", key);
                resultMap.put("type", statistics);
                if ("total".equals(statistics)) {
                    BigDecimal sum = BigDecimal.ZERO;
                    //求和
                    for (String value : valueList) {
                        sum = sum.add(new BigDecimal(value));
                    }
                    resultMap.put("value", sum.toString());
                } else if ("mean".equals(statistics)) {
                    //求平均值
                    BigDecimal sum = BigDecimal.ZERO;
                    for (String value : valueList) {
                        sum = sum.add(new BigDecimal(value));
                    }
                    BigDecimal avg = sum.divide(new BigDecimal(valueList.size()),2,BigDecimal.ROUND_HALF_UP);
                    resultMap.put("value", avg.toString());
                }
                result.add(resultMap);
            }
            statisticalDTO.setResult(result);
            list.add(statisticalDTO);
        }
        return success(list);
    }
}
