package com.xinwangdao.CompanyStatistical.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.base.Strings;
import com.xinwangdao.CompanyStatistical.component.GlobalResponse;
import com.xinwangdao.CompanyStatistical.component.QueryPageBean;
import com.xinwangdao.CompanyStatistical.component.ResultPageBean;
import com.xinwangdao.CompanyStatistical.entity.ProjectDetail;
import com.xinwangdao.CompanyStatistical.entity.SatisfactionModel;
import com.xinwangdao.CompanyStatistical.service.IProjectDetailService;
import com.xinwangdao.CompanyStatistical.service.ISatisfactionModelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.fail;
import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.success;

/**
 *  前端控制器
 *
 * @Author: zthu
 * @Date: 2021-06-15
 */
@Api(description = "")
@Slf4j
@RestController
@RequestMapping("/api/admin/project-detail")
public class ProjectDetailController {
    @Autowired
    IProjectDetailService projectDetailService;

    @Autowired
    ISatisfactionModelService satisfactionModelService;

    @ApiOperation("根据id获取")
    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public GlobalResponse<ProjectDetail> getOne(@PathVariable Integer id) {
        ProjectDetail projectDetail = projectDetailService.getById(id);
        if(3 == projectDetail.getType()){
            SatisfactionModel satisfactionModel = satisfactionModelService.getById(projectDetail.getSatisfactionId());
            projectDetail.setDetailContent(satisfactionModel.getSatisfactionContent());
        }
        return success(projectDetail);
    }

    @ApiOperation("新增")
    @RequestMapping(value = "", method = RequestMethod.POST)
    public GlobalResponse addOne(@RequestBody ProjectDetail entity) {
        if(3 == entity.getType()){
            QueryWrapper<ProjectDetail> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("satisfaction_id",entity.getSatisfactionId())
                    .eq("project_id",entity.getProjectId());
            ProjectDetail one = projectDetailService.getOne(queryWrapper);
            if(null != one){
                return fail("请勿重复添加满意度调查表");
            }
        }
        projectDetailService.save(entity);
        return success(entity);
    }

    @ApiOperation("修改")
    @RequestMapping(value = "/{id}", method = RequestMethod.PUT)
    public GlobalResponse updateOne(@PathVariable Integer id,
                                                @RequestBody ProjectDetail entity) {
        projectDetailService.updateById(entity.setId(id));
        return success(entity);
    }

    @ApiOperation("删除")
    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public GlobalResponse deleteOne(@PathVariable Integer id) {
        projectDetailService.removeById(id);
        return success();
    }

    @ApiOperation("列表查询")
    @RequestMapping(value = "", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<ProjectDetail>> list(QueryPageBean queryBean) {
        IPage page = projectDetailService.page(queryBean.toPage(), Wrappers.<ProjectDetail>lambdaQuery()
                .like(!Strings.isNullOrEmpty(queryBean.getKeyword()), ProjectDetail::getId, queryBean.getKeyword())
        );
        ResultPageBean<ProjectDetail> returnVal = new ResultPageBean(page);
        return success(returnVal);
    }

    @ApiOperation("根据项目id查询列表")
    @RequestMapping(value = "/queryListByProjectId", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<ProjectDetail>> queryListByProjectId(QueryPageBean queryBean,@RequestParam("projectId")Integer projectId,@RequestParam(value = "type",required = false)Integer type) {
        IPage page = null;
        if(null !=type && 2==type){
            page = projectDetailService.page(queryBean.toPage(), Wrappers.<ProjectDetail>lambdaQuery()
                    .eq(ProjectDetail::getProjectId, projectId).eq(ProjectDetail::getType,2)
            );
        }else {
            page = projectDetailService.page(queryBean.toPage(), Wrappers.<ProjectDetail>lambdaQuery()
                    .eq(ProjectDetail::getProjectId, projectId).ne(ProjectDetail::getType,2)
            );
        }
        List<ProjectDetail> records = page.getRecords();
        for (ProjectDetail projectDetail:records){
            if(3 == projectDetail.getType()){
                SatisfactionModel satisfactionModel = satisfactionModelService.getById(projectDetail.getSatisfactionId());
                projectDetail.setDetailContent(satisfactionModel.getSatisfactionContent());
            }
        }
        ResultPageBean<ProjectDetail> returnVal = new ResultPageBean(page);
        return success(returnVal);
    }
}
