package com.xinwangdao.CompanyStatistical.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.base.Strings;
import com.xinwangdao.CompanyStatistical.component.GlobalResponse;
import com.xinwangdao.CompanyStatistical.component.QueryPageBean;
import com.xinwangdao.CompanyStatistical.component.ResultPageBean;
import com.xinwangdao.CompanyStatistical.entity.Project;
import com.xinwangdao.CompanyStatistical.entity.SatisfactionLog;
import com.xinwangdao.CompanyStatistical.entity.SatisfactionModel;
import com.xinwangdao.CompanyStatistical.service.IProjectService;
import com.xinwangdao.CompanyStatistical.service.ISatisfactionLogService;
import com.xinwangdao.CompanyStatistical.service.ISatisfactionModelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.fail;
import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.success;

/**
 *  前端控制器
 *
 * @Author: zthu
 * @Date: 2021-06-15
 */
@Api(description = "")
@Slf4j
@RestController
@RequestMapping("/api/admin/satisfaction-log")
public class SatisfactionLogController {
    @Autowired
    ISatisfactionLogService satisfactionLogService;

    @Autowired
    ISatisfactionModelService satisfactionModelService;

    @Autowired
    IProjectService projectService;

    @ApiOperation("根据id获取")
    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public GlobalResponse<SatisfactionLog> getOne(@PathVariable Integer id) {
        return success(satisfactionLogService.getById(id));
    }

    @ApiOperation("新增")
    @RequestMapping(value = "", method = RequestMethod.POST)
    public GlobalResponse addOne(@RequestBody SatisfactionLog entity) {
        Project project = projectService.getById(entity.getProjectId());
        if(1 != project.getStatus()){
            return fail("项目已停用或已截至");
        }
        SatisfactionModel model = satisfactionModelService.getById(entity.getSatisfactionId());
        if(1 == model.getFlag()){
            QueryWrapper<SatisfactionLog> satisfactionLogQueryWrapper = new QueryWrapper<>();
            satisfactionLogQueryWrapper.eq("satisfaction_id",entity.getSatisfactionId()).eq("company_id",entity.getCompanyId()).eq("project_id",entity.getProjectId())
                    .eq("status",1);
            SatisfactionLog satisfactionLog = satisfactionLogService.getOne(satisfactionLogQueryWrapper);
            if(null != satisfactionLog){
                return fail("您已提交该项目的满意度表单，请勿重复提交");
            }
        }else {
//            QueryWrapper<SatisfactionLog> satisfactionLogQueryWrapper1 = new QueryWrapper<>();
//            satisfactionLogQueryWrapper1.eq("satisfaction_id",entity.getSatisfactionId()).eq(!Strings.isNullOrEmpty(entity.getNickName()),"nick_name",entity.getNickName());
//            SatisfactionLog satisfactionLog1 = satisfactionLogService.getOne(satisfactionLogQueryWrapper1);
//            if(null != satisfactionLog1){
//                return fail("您已提交该项目的满意度表单，请勿重复提交");
//            }
        }

        entity.setCreateTime(new Date());
        satisfactionLogService.save(entity);
        return success(entity);
    }

    @ApiOperation("修改")
    @RequestMapping(value = "/{id}", method = RequestMethod.PUT)
    public GlobalResponse updateOne(@PathVariable Integer id,
                                                @RequestBody SatisfactionLog entity) {
        SatisfactionLog satisfactionLog = satisfactionLogService.getById(id);
        Project project = projectService.getById(satisfactionLog.getProjectId());
        if(1 != project.getStatus()){
            return fail("项目已停用或已截至");
        }
        satisfactionLogService.updateById(entity.setId(id));
        return success(entity);
    }

    @ApiOperation("删除")
    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public GlobalResponse deleteOne(@PathVariable Integer id) {
        satisfactionLogService.removeById(id);
        return success();
    }

    @ApiOperation("列表查询")
    @RequestMapping(value = "", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<SatisfactionLog>> list(QueryPageBean queryBean) {
        IPage page = satisfactionLogService.page(queryBean.toPage(), Wrappers.<SatisfactionLog>lambdaQuery()
                .like(!Strings.isNullOrEmpty(queryBean.getKeyword()), SatisfactionLog::getId, queryBean.getKeyword())
        );
        ResultPageBean<SatisfactionLog> returnVal = new ResultPageBean(page);
        return success(returnVal);
    }

    @ApiOperation("根据满意度模板id查询提交log列表")
    @RequestMapping(value = "/queryLogListBySatisfactionId", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<SatisfactionLog>> queryLogListBySatisfactionId(QueryPageBean queryBean,
                                                                                            @RequestParam(value = "satisfactionId")Integer satisfactionId) {
        IPage page = satisfactionLogService.page(queryBean.toPage(), Wrappers.<SatisfactionLog>lambdaQuery()
                .eq(null != satisfactionId, SatisfactionLog::getSatisfactionId, satisfactionId)
        );
        ResultPageBean<SatisfactionLog> returnVal = new ResultPageBean(page);
        return success(returnVal);
    }
}
