package com.xinwangdao.CompanyStatistical.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.Strings;
import com.xinwangdao.CompanyStatistical.component.GlobalResponse;
import com.xinwangdao.CompanyStatistical.component.QueryPageBean;
import com.xinwangdao.CompanyStatistical.component.ResultPageBean;
import com.xinwangdao.CompanyStatistical.entity.SatisfactionModel;
import com.xinwangdao.CompanyStatistical.entity.dto.SatisfactionDTO;
import com.xinwangdao.CompanyStatistical.entity.dto.SatisfactionModelDTO;
import com.xinwangdao.CompanyStatistical.entity.dto.ScoreDTO;
import com.xinwangdao.CompanyStatistical.mapper.SatisfactionModelMapper;
import com.xinwangdao.CompanyStatistical.service.ISatisfactionModelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.xinwangdao.CompanyStatistical.component.GlobalResponse.success;

/**
 *  前端控制器
 *
 * @Author: zthu
 * @Date: 2021-06-15
 */
@Api(description = "")
@Slf4j
@RestController
@RequestMapping("/api/admin/satisfaction-model")
public class SatisfactionModelController {
    @Autowired
    ISatisfactionModelService satisfactionModelService;

    @Autowired
    SatisfactionModelMapper satisfactionModelMapper;

    @ApiOperation("根据id获取")
    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public GlobalResponse<SatisfactionModel> getOne(@PathVariable Integer id) {
        return success(satisfactionModelService.getById(id));
    }

    @ApiOperation("新增")
    @RequestMapping(value = "", method = RequestMethod.POST)
    public GlobalResponse addOne(@RequestBody SatisfactionModel entity) {
        entity.setCreateTime(new Date());
        satisfactionModelService.save(entity);
        return success(entity);
    }

    @ApiOperation("修改")
    @RequestMapping(value = "/{id}", method = RequestMethod.PUT)
    public GlobalResponse updateOne(@PathVariable Integer id,
                                                @RequestBody SatisfactionModel entity) {
        satisfactionModelService.updateById(entity.setId(id));
        return success(entity);
    }

    @ApiOperation("删除")
    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public GlobalResponse deleteOne(@PathVariable Integer id) {
        satisfactionModelService.removeById(id);
        return success();
    }

    @ApiOperation("列表查询")
    @RequestMapping(value = "", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<SatisfactionModelDTO>> list(QueryPageBean queryBean,@RequestParam(value = "flag",required = false)Integer flag,
                                                                     @RequestParam(value = "name",required = false)String name) {
        IPage page = satisfactionModelService.page(queryBean.toPage(), Wrappers.<SatisfactionModel>lambdaQuery()
                .eq(null != flag, SatisfactionModel::getFlag, flag).like(!Strings.isNullOrEmpty(name),SatisfactionModel::getSatisfactionName,name).orderByDesc(SatisfactionModel::getId)
        );

        List<SatisfactionModel> records = page.getRecords();
        ResultPageBean<SatisfactionModelDTO> returnVal = new ResultPageBean(page);
        List<SatisfactionModelDTO> satisfactionModelDTOList = new ArrayList<>();
        if(null != records && records.size()>0) {
            for (SatisfactionModel model:records) {
                ScoreDTO scoreDTO = satisfactionModelMapper.selectCountNum(model.getId());
                SatisfactionModelDTO dto = new SatisfactionModelDTO();
                BeanUtils.copyProperties(model,dto);
                dto.setTotalGoalScore(scoreDTO.getSocreNum());
                dto.setTotalEntireScore(new BigDecimal(scoreDTO.getTotalNum()).multiply(model.getTotalScore()));
                satisfactionModelDTOList.add(dto);
            }
        }
        returnVal.setRecords(satisfactionModelDTOList);
        return success(returnVal);
    }

    @ApiOperation("根据满意度表单查询企业列表")
    @RequestMapping(value = "/queryCompanyListBySatisfactionId", method = RequestMethod.GET)
    public GlobalResponse<ResultPageBean<SatisfactionDTO>> queryCompanyListBySatisfactionId(QueryPageBean queryBean,
                                                                                              @RequestParam(value = "satisfactionId")Integer satisfactionId,
                                                                                            @RequestParam(value = "projectName",required = false)String projectName,
                                                                                            @RequestParam(value = "companyName",required = false)String companyName) {
        Page<SatisfactionModel> page1 = new Page<>(queryBean.getCurrent(), queryBean.getSize());
        IPage page = satisfactionModelService.queryCompanyListBySatisfactionId(page1,satisfactionId,companyName,projectName);
        ResultPageBean<SatisfactionDTO> returnVal = new ResultPageBean(page);
        return success(returnVal);
    }


}
