package com.xinwangdao.CompanyStatistical.controller;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;


/**
 * @Author: zthu
 * @Date: 2021-06-15
 */
@Component
@Slf4j
@ServerEndpoint(value = "/api/websocket")
public class WebSocketController {
    private static Map<String, Session> onlineSessions = new ConcurrentHashMap<>();

    @OnOpen
    public void onOpen(Session session) {
        onlineSessions.put(session.getId(), session);
        log.info("websocket - [{}] connect", session.getId());
    }

    @OnMessage
    public void onMessage(Session session, String jsonStr) {
        log.info("websocket - [{}] => {}", session.getId(), jsonStr);
    }

    @OnClose
    public void onClose(Session session) {
        onlineSessions.remove(session.getId());
        log.info("websocket - [{}] close", session.getId());
    }

    @OnError
    public void onError(Session session, Throwable error) {
        error.printStackTrace();
        log.error("websocket - [{}] error", session.getId());
    }

    public static void sendMessage(String id, String msg) {
        try {
            onlineSessions.get(id).getBasicRemote().sendText(msg);
            log.info("websocket - {} => [{}]", msg, id);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void sendMessageToAll(String msg) {
        log.info("websocket - [All user] => {}", msg);
        onlineSessions.forEach((id, session) -> {
            try {
                session.getBasicRemote().sendText(msg);
            } catch (IOException e) {
                e.printStackTrace();
            }
        });
    }

}

