package com.xinwangdao.CompanyStatistical.handler;

import com.xinwangdao.CompanyStatistical.component.GlobalException;
import com.xinwangdao.CompanyStatistical.component.GlobalResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author: zthu
 * @Date: 2021-06-15
 */
@Slf4j
@ControllerAdvice
@Order(1)
public class RequestExceptionHandler {
    /**
     * @param ex
     * @return
     * @Valid 注解异常
     */
    @ResponseBody
    @ResponseStatus(HttpStatus.OK)
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public GlobalResponse QueryValidExceptionHandler(MethodArgumentNotValidException ex) {
        log.error("MethodArgumentNotValidException catch => " + ex);
        String errMsg = null;
        BindingResult result = ex.getBindingResult();
        if (result.hasErrors()) {
            List<ObjectError> errors = result.getAllErrors();
            errMsg = errors.stream().map(i -> String.format("[%s]%s", ((FieldError) i).getField(), i.getDefaultMessage())).collect(Collectors.joining(";"));
        }
        return GlobalResponse.fail(GlobalException.REQUEST_PARAMS_INVALID, errMsg);
    }
}
