package com.xinwangdao.CompanyStatistical.handler;

import com.xinwangdao.CompanyStatistical.component.GlobalException;
import com.xinwangdao.CompanyStatistical.component.GlobalResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.apache.shiro.authc.UnknownAccountException;
import org.apache.shiro.authz.UnauthenticatedException;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * @Author: zthu
 * @Date: 2021-06-15
 */
@Slf4j
@ControllerAdvice
@Order(2)
public class UserExceptionHandler {
    @ResponseBody
    @ResponseStatus(HttpStatus.OK)
    @ExceptionHandler(UnknownAccountException.class)
    public GlobalResponse UnknownAccountExceptionHandler(UnknownAccountException ex) {
        log.error("UnknownAccountException catch => " + ex);
        return GlobalResponse.fail(GlobalException.USER_NOT_EXIST);
    }

    @ResponseBody
    @ResponseStatus(HttpStatus.OK)
    @ExceptionHandler({IncorrectCredentialsException.class})
    public GlobalResponse IncorrectCredentialsExceptionHandler(IncorrectCredentialsException ex) {
        log.error("IncorrectCredentialsException catch => " + ex);
        return GlobalResponse.fail(GlobalException.USER_PASSWORD_INVALID);
    }

    @ResponseBody
    @ResponseStatus(HttpStatus.OK)
    @ExceptionHandler({UnauthenticatedException.class})
    public GlobalResponse UnauthenticatedExceptionHandler(UnauthenticatedException ex) {
        log.error("UnauthenticatedException catch => " + ex);
        if (ex.getMessage().contains("The current Subject is not a user"))
            return GlobalResponse.fail(GlobalException.USER_NOT_LOGIN);
        if (ex.getMessage().contains("The current Subject is not a guest"))
            return GlobalResponse.fail(GlobalException.USER_ALREADY_LOGIN);
        return GlobalResponse.fail(GlobalException.ACCESS_DENIED);
    }


}
